package ai.pai.ptp.test;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.view.View;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.ListView;

import com.remoteyourcam.usb.ptp.Camera;
import com.remoteyourcam.usb.ptp.PtpConstants;
import com.remoteyourcam.usb.ptp.PtpService;
import com.remoteyourcam.usb.ptp.model.LiveViewData;
import com.remoteyourcam.usb.ptp.model.ObjectInfo;

import ai.pai.ptp.R;


public class MyTestActivity extends Activity implements Camera.CameraListener, Camera.StorageInfoListener, Camera.RetrieveImageInfoListener {

    private ImageView photoIv;
    private TestLogListAdapter adapter;
    private GridView photoGridView;
    private TestPhotoGridAdapter photoGridAdapter;
    private PtpService ptp;
    private Camera camera;
    private boolean isInStart;
    private int mode = 0;  //0 拍照新增模式 1 照片浏览模式

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_test);

        ptp = PtpService.Singleton.getInstance(this);

        photoIv = (ImageView) findViewById(R.id.iv_latest_photo);
        ListView infoListView = (ListView) findViewById(R.id.list_info);
        adapter = new TestLogListAdapter(this, infoListView);
        infoListView.setAdapter(adapter);

        photoGridView = (GridView) findViewById(R.id.gridview_photos);
        photoGridAdapter = new TestPhotoGridAdapter(this, photoGridView);
        photoGridView.setAdapter(photoGridAdapter);

        findViewById(R.id.btn_get_devece_info).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (camera != null) {
                    adapter.addInfo(camera.getDeviceInfo());
                } else {
                    adapter.addInfo("未发现相机设备");
                    ptp.initialize(MyTestActivity.this, getIntent());
                }
            }
        });

        findViewById(R.id.btn_capture_photo).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (camera != null) {
                    mode = 0;
                    photoGridAdapter.clear();
                    camera.capture();
                } else {
                    ptp.initialize(MyTestActivity.this, getIntent());
                }
            }
        });

        findViewById(R.id.btn_get_photos).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (camera != null) {
                    mode = 1;
                    photoGridAdapter.clear();
                    camera.retrieveImageHandles(MyTestActivity.this, 0xFFFFFFFF, PtpConstants.ObjectFormat.EXIF_JPEG);
                } else {
                    ptp.initialize(MyTestActivity.this, getIntent());
                }
            }
        });
    }

    @Override
    protected void onStart() {
        super.onStart();
        isInStart = true;
        ptp.setCameraListener(this);
        ptp.initialize(this, getIntent());
    }

    @Override
    protected void onStop() {
        super.onStop();
        isInStart = false;
        ptp.setCameraListener(null);
        if (isFinishing()) {
            ptp.shutdown();
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        this.setIntent(intent);
        if (isInStart) {
            ptp.initialize(this, intent);
        }
    }

    @Override
    public void onCameraStarted(Camera camera) {
        this.camera = camera;
        adapter.addInfo("开始会话");

    }

    @Override
    public void onCameraStopped(Camera camera) {
        this.camera = null;
        adapter.addInfo("结束会话");
    }

    @Override
    public void onNoCameraFound() {
        adapter.addInfo("没有发现相机设备");
    }

    @Override
    public void onError(String message) {
        adapter.addInfo("发生错误 " + message + "\n请重新打开相机");
        camera = null;
    }

    @Override
    public void onPropertyChanged(int property, int value) {

    }

    @Override
    public void onPropertyStateChanged(int property, boolean enabled) {

    }

    @Override
    public void onPropertyDescChanged(int property, int[] values) {

    }

    @Override
    public void onLiveViewStarted() {

    }

    @Override
    public void onLiveViewData(LiveViewData data) {

    }

    @Override
    public void onLiveViewStopped() {

    }

    @Override
    public void onCapturedPictureReceived(int objectHandle, String filename, Bitmap thumbnail, Bitmap bitmap) {
        adapter.addInfo("收到新照片 ");
        Photo photo = new Photo();
        photo.handle = objectHandle;
        photo.fileName = filename;
        photo.thumb = thumbnail;
        photo.origin = bitmap;
        photo.captureTime = System.currentTimeMillis() + "";
        photoGridAdapter.addPhoto(photo);
    }

    @Override
    public void onBulbStarted() {

    }

    @Override
    public void onBulbExposureTime(int seconds) {

    }

    @Override
    public void onBulbStopped() {

    }

    @Override
    public void onFocusStarted() {

    }

    @Override
    public void onFocusEnded(boolean hasFocused) {

    }

    @Override
    public void onFocusPointsChanged() {

    }

    @Override
    public void onObjectAdded(int handle, int format) {
        adapter.addInfo("相机系统发现新文件 ,文件句柄 =" + handle + " format = " + format);
        if (camera == null) {
            return;
        }
        if (format == PtpConstants.ObjectFormat.EXIF_JPEG) {
            camera.retrievePicture(handle);
        }
    }

    @Override
    public void onLogMessage(String msg) {
        adapter.addInfo(msg);
    }

    @Override
    public void onStorageFound(final int handle, String label) {
        adapter.addInfo("找到相机存储卡位置，句柄=" + handle + " label = " + label);
//        photoIv.post(new Runnable() {
////            @Override
////            public void run() {
////                camera.retrieveImageHandles(MyTestActivity.this, handle, PtpConstants.ObjectFormat.EXIF_JPEG);
////            }
////        });
    }

    @Override
    public void onAllStoragesFound() {
        adapter.addInfo("相机存储卡查找完成");
    }

    @Override
    public void onImageHandlesRetrieved(final int[] handles) {
        adapter.addInfo("找到相机存储图像文件，共" + handles.length + "张");
        photoIv.post(new Runnable() {
            @Override
            public void run() {
                photoGridAdapter.addPhotos(handles);
            }
        });
    }

    @Override
    public void onImageInfoRetrieved(final int objectHandle, final ObjectInfo objectInfo, final Bitmap thumbnail) {
        adapter.addInfo("找到相机存储图像文件，句柄=" + objectHandle + " 文件名 = " + objectInfo.filename + " 拍摄日期 = "+ objectInfo.captureDate);
        photoIv.post(new Runnable() {
            @Override
            public void run() {
                Photo photo = new Photo();
                photo.handle = objectHandle;
                photo.fileName = objectInfo.filename;
                photo.thumb = thumbnail;
                photo.captureTime = objectInfo.captureDate;
                photoGridAdapter.updatePhoto(photo);
            }
        });
    }

    public void retrieveImageInfo(final int handle) {
        photoIv.post(new Runnable() {
            @Override
            public void run() {
                if (camera != null) {
                    camera.retrieveImageInfo(MyTestActivity.this, handle);
                }
            }
        });

    }
}
